@@ -9,7 +9,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2012 Pascal Buchbinder
+ * Copyright (C) 2007-2013 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,13 +28,14 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.52 2012/09/20 15:19:21 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.61 2013/06/12 19:05:25 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <errno.h>
 #include <stdarg.h>
+#include <ctype.h>
 
 #include <stdlib.h>
 #include <unistd.h>
@@ -48,6 +49,7 @@ static const char revision[] = "$Id: qslog.c,v 1.52 2012/09/20 15:19:21 pbuchbin
 #include <apr.h>
 #include <apr_portable.h>
 #include <apr_support.h>
+#include <apr_strings.h>
 
 #include "qs_util.h"
 
@@ -60,6 +62,8 @@ static const char revision[] = "$Id: qslog.c,v 1.52 2012/09/20 15:19:21 pbuchbin
 #define RULE_DELIM ':'
 #define MAX_CLIENT_ENTRIES 25000
 #define QS_GENERATIONS 14
+#define EVENT_DELIM ','
+#define QSEVENTPATH "QSEVENTPATH" /* varibale name to find event definitions */
 
 /* ----------------------------------
  * structures
@@ -82,6 +86,8 @@ typedef struct {
   long status_4;
   long status_5;
   long connections;
+  apr_table_t *events;
+  apr_pool_t *pool;
 } client_rec_t;
 
 typedef struct stat_rec_st {
@@ -107,7 +113,9 @@ typedef struct stat_rec_st {
 
   unsigned long long sum;
   unsigned long long average;
+  long average_count;
   unsigned long long averAge;
+  long averAge_count;
 
   long status_1;
   long status_2;
@@ -122,6 +130,9 @@ typedef struct stat_rec_st {
   long qos_t;
   long qos_l;
   long qos_ser;
+
+  apr_table_t *events;
+  apr_pool_t *pool;
 } stat_rec_t;
 
 /* ----------------------------------
@@ -238,7 +249,7 @@ static void qsNoFloat(char *s) {
 static void stripNum(char **p) {
   char *s = *p;
   int len;
-  while(s && (s[0] < '0' || s[0] > '9')) {
+  while(s && s[0] && (s[0] < '0' || s[0] > '9')) {
     s++;
   }
   len = strlen(s);
@@ -405,8 +416,8 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
     // max len: 18446744073709551615
     sprintf(custom, "s;%llu;a;%llu;A;%llu;",
             stat_rec->sum,
-            stat_rec->average / (stat_rec->line_count == 0 ? 1 : stat_rec->line_count),
-            stat_rec->averAge / (stat_rec->line_count == 0 ? 1 : stat_rec->line_count));
+            stat_rec->average / (stat_rec->average_count == 0 ? 1 : stat_rec->average_count),
+            stat_rec->averAge / (stat_rec->averAge_count == 0 ? 1 : stat_rec->averAge_count));
   }
   if(main) {
     sprintf(ip, "ip;%ld;", qs_countEvent(&m_ip_list));
@@ -490,7 +501,9 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   }
   stat_rec->sum = 0;
   stat_rec->average = 0;
+  stat_rec->average_count = 0;
   stat_rec->averAge = 0;
+  stat_rec->averAge_count = 0;
   stat_rec->status_1 = 0;
   stat_rec->status_2 = 0;
   stat_rec->status_3 = 0;
@@ -514,16 +527,97 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
   stat_rec->qos_ser = 0;
   if(main) {
     if(!offline) {
-      fprintf(f, "sl;%.2f;m;%s",
+      fprintf(f, "sl;%.2f;m;%s;",
               av[0], mem[0] ? mem : "-");
     } else {
-      fprintf(f, "sl;-;m;-");
+      fprintf(f, "sl;-;m;-;");
       m_offline_data = 0;
     }
   }
+  if(apr_table_elts(stat_rec->events)->nelts > 0) {
+    int i;
+    apr_table_entry_t *entry = (apr_table_entry_t *) apr_table_elts(stat_rec->events)->elts;
+    for(i = 0; i < apr_table_elts(stat_rec->events)->nelts; i++) {
+      const char *eventName = entry[i].key;
+      int *eventVal = (int *)entry[i].val;
+      fprintf(f, "%s;%d;", eventName, *eventVal);
+      (*eventVal) = 0;
+    }
+  }
   fprintf(f, "\n");
 }
 
+static void qs_updateEvents(apr_pool_t *pool, char *E, apr_table_t *events) {
+  if(!E[0]) {
+    return;
+  }
+  while(E) {
+    char *restore = NULL;
+    char *sep = strchr(E, EVENT_DELIM);
+    int *val;
+    if(sep) {
+      sep[0] = '\0';
+      restore = sep;
+      sep++;
+    }
+    if(isalnum(E[0])) {
+      val = (int *)apr_table_get(events, E);
+      if(val) {
+        (*val)++;
+      } else {
+        // new event
+        char *name = apr_pstrdup(pool, E);
+        val = apr_pcalloc(pool, sizeof(int));
+        (*val) = 1;
+        apr_table_setn(events, name, (char *)val);
+      }
+    }
+    E = sep;
+    if(restore) {
+      // suports multiple parsing of the event string
+      restore[0] = EVENT_DELIM;
+    }
+  }
+}
+
+/**
+ * Initializes the event table by the events specified within the
+ * file whose path is defined by the QSEVENTPATH environment
+ * variable.
+ *
+ * @param pool To allocate memory
+ * @param events Table to init
+ */
+static void qsInitEvent(apr_pool_t *pool, apr_table_t *events) {
+  const char *envFile = getenv(QSEVENTPATH);
+  if(envFile != NULL) {
+    FILE *file = fopen(envFile, "r"); 
+    if(file != NULL) {
+      char line[MAX_LINE];
+      while(!qs_getLinef(line, sizeof(line), file)) {
+        char *p = line;
+        char *name;
+        int *val;
+        while(p && p[0]) {
+          /* file contains a list of known events (comma sep.
+             event names on one or multiple lines) */
+          char *n = strchr(p, EVENT_DELIM);
+          if(n) {
+            n[0] = '\0';
+            n++;
+          }
+          name = apr_pstrdup(pool, p);
+          val = apr_pcalloc(pool, sizeof(int));
+          (*val) = 0;
+          apr_table_setn(events, name, (char *)val);
+          p = n;
+        }
+      }
+      fclose(file);
+    }
+  }
+}
+
 /**
  * Creates and init new status rec
  *
@@ -531,7 +625,7 @@ static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec,
  * @param pattern Pattern to match the log data line
  * @return
  */
-static stat_rec_t *createRec(const char *id, const char *pattern) {
+static stat_rec_t *createRec(apr_pool_t *pool, const char *id, const char *pattern) {
   stat_rec_t *rec = calloc(sizeof(stat_rec_t), 1);
   rec->id = calloc(strlen(id)+2, 1);
   sprintf(rec->id, "%s;", id);
@@ -558,7 +652,9 @@ static stat_rec_t *createRec(const char *id, const char *pattern) {
 
   rec->sum = 0;
   rec->average = 0;
+  rec->average_count = 0;
   rec->averAge = 0;
+  rec->averAge_count = 0;
 
   rec->status_1 = 0;
   rec->status_2 = 0;
@@ -573,6 +669,10 @@ static stat_rec_t *createRec(const char *id, const char *pattern) {
   rec->qos_t = 0;
   rec->qos_l = 0;
   rec->qos_ser = 0;
+
+  rec->events = apr_table_make(pool, 300);
+  rec->pool = pool;
+  qsInitEvent(pool, rec->events);
   return rec;
 }
 
@@ -637,9 +737,9 @@ static void printAndResetStat(char *timeStr) {
 /**
  * Updates the per client record
  */
-static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
+static void updateClient(apr_pool_t *pool, char *T, char *t, char *D, char *S,
                          char *BI, char *B, char *R, char *I, char *U, char *Q,
-                         char *k, char *C, long tme) {
+                         char *E, char *k, char *C, long tme) {
   client_rec_t *client_rec;
   const char *id = I; // ip
   if(id == NULL) {
@@ -679,6 +779,9 @@ static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
     client_rec->status_4 = 0;
     client_rec->status_5 = 0;
     client_rec->connections = 0;
+    client_rec->events = apr_table_make(pool, 100);
+    client_rec->pool = pool;
+    qsInitEvent(pool, client_rec->events);
     apr_table_setn(m_client_entries, tid, (char *)client_rec);
   }
   client_rec->request_count++;
@@ -721,6 +824,9 @@ static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
       client_rec->status_5++;
     }
   }
+  if(E != NULL) {
+    qs_updateEvents(client_rec->pool, E, client_rec->events);
+  }
   return;
 }
 
@@ -730,7 +836,7 @@ static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
 static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
                       char *s, char *a, char *A,
                       char *BI, char *B, char *R, char *I, char *U, char *Q,
-                      char *k, char *C, long tme, long tmems) {
+                      char *E, char *k, char *C, long tme, long tmems) {
   if(Q != NULL) {
     if(strchr(Q, 'V') != NULL) {
       rec->qos_v++;
@@ -754,6 +860,9 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
       rec->qos_ser++;
     }
   }
+  if(E != NULL) {
+    qs_updateEvents(rec->pool, E, rec->events);
+  }
   if(I != NULL) {
     /* update/store client IP */
     qs_insertEvent(&m_ip_list, I);
@@ -778,11 +887,13 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
   if(s != NULL) {
     rec->sum += atol(s);
   }
-  if(a != NULL) {
+  if(a != NULL && a[0]) {
     rec->average += atol(a);
+    rec->average_count++;
   }
-  if(A != NULL) {
+  if(A != NULL && A[0]) {
     rec->averAge += atol(A);
+    rec->averAge_count++;
   }
   if(S != NULL) {
     if(S[0] == '1') {
@@ -835,7 +946,7 @@ static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
  * 127.0.0.1 [03/Nov/2006:21:06:41 +0100] "GET /index.html HTTP/1.1" 200 2836 "Wget/1.9.1" 0
  * .         .                     .      R                                   .            T
  */
-static void updateStat(const char *cstr, char *line) {
+static void updateStat(apr_pool_t *pool, const char *cstr, char *line) {
   stat_rec_t *rec = NULL;
   char *T = NULL; /* time */
   char *t = NULL; /* time ms */
@@ -852,6 +963,7 @@ static void updateStat(const char *cstr, char *line) {
   char *s = NULL; /* sum */
   char *a = NULL; /* avarage 1 */
   char *A = NULL; /* average 2 */
+  char *E = NULL; /* events */
   const char *c = cstr;
   char *l = line;
   long tme;
@@ -923,10 +1035,14 @@ static void updateStat(const char *cstr, char *line) {
       if(l != NULL && l[0] != '\0') {
         A = cutNext(&l);
       }
+    } else if(c[0] == 'E') {
+      if(l != NULL && l[0] != '\0') {
+        E = cutNext(&l);
+      }
     } else if(c[0] == ' ') {
       /* do nothing */
     } else {
-      /* undedined char, skip it */
+      /* undefined/unknown char, skip it */
       if(l != NULL && l[0] != '\0') {
         l++;
       }
@@ -977,16 +1093,18 @@ static void updateStat(const char *cstr, char *line) {
       tme = tmems / 1000;
     } else if(D) {
       stripNum(&D);
-      tmems = atol(D) / 1000;
+      tmems = atol(D);
+      tmems = tmems / 1000;
       tme = tmems / 1000;
     }
   }
-  updateRec(m_stat_rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, k, C, tme, tmems);
-  if(rec) {
-    updateRec(rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, k, C, tme, tmems);
-  }
-  if(m_offline_count) {
-    updateClient(NULL, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
+  if(!m_offline_count) {
+    updateRec(m_stat_rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
+    if(rec) {
+      updateRec(rec, T, t, D, S, s, a, A, BI, B, R, I, U, Q, E, k, C, tme, tmems);
+    }
+  } else {
+    updateClient(pool, T, t, D, S, BI, B, R, I, U, Q, E, k, C, tme);
   }
   qs_csUnLock();
 
@@ -1107,7 +1225,7 @@ static time_t getMinutes(char *line) {
  * reads from stdin and calls updateStat()
  * => used for real time analysis
  */
-static void readStdin(const char *cstr) {
+static void readStdin(apr_pool_t *pool, const char *cstr) {
   char line[MAX_LINE];
   int line_len;
   while(fgets(line, sizeof(line), stdin) != NULL) {
@@ -1119,7 +1237,7 @@ static void readStdin(const char *cstr) {
       line[line_len] = '\0';
       line_len--;
     }
-    updateStat(cstr, line);
+    updateStat(pool, cstr, line);
   }
 }
 
@@ -1130,7 +1248,7 @@ static void readStdin(const char *cstr) {
  * access log lines
  * => used for offline analysis
  */
-static void readStdinOffline(const char *cstr) {
+static void readStdinOffline(apr_pool_t *pool, const char *cstr) {
   char line[MAX_LINE];
   char buf[32];
   time_t unitTime = 0;
@@ -1152,10 +1270,10 @@ static void readStdinOffline(const char *cstr) {
       qs_setTime(unitTime * 60);
     }
     if(unitTime == l_time) {
-      updateStat(cstr, line);
+      updateStat(pool, cstr, line);
     } if(l_time < unitTime) {
       /* leap in time... */
-      updateStat(cstr, line);
+      updateStat(pool, cstr, line);
       fprintf(stdout, "X");
       fflush(stdout);
       unitTime = 0;
@@ -1174,7 +1292,7 @@ static void readStdinOffline(const char *cstr) {
         unitTime++;
         qs_setTime(unitTime * 60);;
       }
-      updateStat(cstr, line);
+      updateStat(pool, cstr, line);
     }
   }
   if(m_offline_data) {
@@ -1306,6 +1424,7 @@ static void usage(const char *cmd, int man) {
   qs_man_println(man, "     s arbitraty counter to add up (sum within a minute)\n");
   qs_man_println(man, "     a arbitraty counter to build an average from (average per request)\n");
   qs_man_println(man, "     A arbitraty counter to build an average from (average per request)\n");
+  qs_man_println(man, "     E comma separated list of event strings\n");
   qs_man_println(man, "     . defines an element to ignore (unknown string)\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -o <out_file>\n");
@@ -1355,7 +1474,7 @@ static void usage(const char *cmd, int man) {
     printf("Example configuration using pipped logging:\n");
   }
   qs_man_println(man, "  LogFormat \"%%t %%h \\\"%%r\\\" %%>s %%b \\\"%%{User-Agent}i\\\" %%T\"\n");
-  qs_man_println(man, "  TransferLog \"|./bin/%s -f ..IRSB.T -x -o ./logs/stat_log\"\n", cmd);
+  qs_man_println(man, "  TransferLog \"|/bin/%s -f ..IRSB.T -x -o /var/logs/stat_log\"\n", cmd);
   printf("\n");
   if(man) {
     printf("Configuration using the CustomLog directive:\n");
@@ -1363,7 +1482,7 @@ static void usage(const char *cmd, int man) {
   } else {
    printf("Example configuration using the CustomLog directive:\n");
   }
-  qs_man_println(man, "  CustomLog \"|./bin/%s -f ISBTQ -x -o ./logs/stat_log\" \"%%h %%>s %%b %%T %%{mod_qos_ev}e\"\n", cmd);
+  qs_man_println(man, "  CustomLog \"|/bin/%s -f ISBTQ -x -o /var/logs/stat_log\" \"%%h %%>s %%b %%T %%{mod_qos_ev}e\"\n", cmd);
   printf("\n");
   if(man) {
     printf("Post processing:\n");
@@ -1371,7 +1490,7 @@ static void usage(const char *cmd, int man) {
   } else {
     printf("Example for post processing:\n");
   }
-  qs_man_println(man, "  cat access_log | ./bin/%s -f ..IRSB.T -o ./logs/stat_log -p\n", cmd);
+  qs_man_println(man, "  cat access_log | /bin/%s -f ..IRSB.T -o /var/logs/stat_log -p\n", cmd);
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
@@ -1394,7 +1513,7 @@ static void usage(const char *cmd, int man) {
  * @param confFile Path to the rule file to load
  * @return
  */
-static stat_rec_t *loadRule(const char *confFile) {
+static stat_rec_t *loadRule(apr_pool_t *pool, const char *confFile) {
   char line[MAX_LINE];
   FILE *file = fopen(confFile, "r"); 
   stat_rec_t *rec = NULL;
@@ -1413,7 +1532,7 @@ static stat_rec_t *loadRule(const char *confFile) {
       if(m_verbose) {
         printf("load rule %s: %s\n", id, p);
       }
-      next = createRec(id, p);
+      next = createRec(pool, id, p);
       if(rec == NULL) {
         // first record
         rec = next;
@@ -1441,7 +1560,10 @@ int main(int argc, const char *const argv[]) {
   const char *username = NULL;
   pthread_attr_t *tha = NULL;
   pthread_t tid;
-  m_stat_rec = createRec("", "");
+  apr_pool_t *pool;
+  apr_app_initialize(&argc, &argv, NULL);
+  apr_pool_create(&pool, NULL);
+  m_stat_rec = createRec(pool, "", "");
 
   qs_csInitLock();
   qs_setExpiration(ACTIVE_TIME);
@@ -1532,7 +1654,7 @@ int main(int argc, const char *const argv[]) {
   }
 
   /*
-   * offline count mode creates statitis
+   * offline count mode creates statistics
    * on a per client basis (e.g. per source
    * ip or user id using the user tracking
    * feature of mod_qos)
@@ -1540,13 +1662,10 @@ int main(int argc, const char *const argv[]) {
   if(m_offline_count) {
     int i;
     apr_table_entry_t *entry;
-    apr_pool_t *pool;
     nice(10);
     if(config == NULL) usage(cmd, 0);
-    apr_app_initialize(&argc, &argv, NULL);
-    apr_pool_create(&pool, NULL);
     m_client_entries = apr_table_make(pool, MAX_CLIENT_ENTRIES);
-    readStdinOffline(config);
+    readStdinOffline(pool, config);
     fprintf(stdout, ".\n");
     entry = (apr_table_entry_t *) apr_table_elts(m_client_entries)->elts;
     m_f = stdout;
@@ -1565,8 +1684,7 @@ int main(int argc, const char *const argv[]) {
       }
       fprintf(m_f, "%s;req;%ld;errors;%ld;"
               "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;"
-              "av;%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;%s"
-              "\n",
+              "av;%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;%s",
               entry[i].key,
               client_rec->request_count,
               client_rec->error_count,
@@ -1584,6 +1702,17 @@ int main(int argc, const char *const argv[]) {
               client_rec->duration_5,
               client_rec->duration_6,
               esco);
+      if(apr_table_elts(client_rec->events)->nelts > 0) {
+        int k;
+        apr_table_entry_t *client_entry = (apr_table_entry_t *) apr_table_elts(client_rec->events)->elts;
+        for(k = 0; k < apr_table_elts(client_rec->events)->nelts; k++) {
+          const char *eventName = client_entry[k].key;
+          int *eventVal = (int *)client_entry[k].val;
+          fprintf(m_f, "%s;%d;", eventName, *eventVal);
+          (*eventVal) = 0;
+        }
+      }
+      fprintf(m_f, "\n");
     }
     if(file && m_f != stdout) {
       fclose(m_f);
@@ -1632,7 +1761,7 @@ int main(int argc, const char *const argv[]) {
       qerror("you need to add 'C' to the format string when enabling the pattern list (-c)");
       exit(1);
     }
-    m_stat_sub = loadRule(confFile);
+    m_stat_sub = loadRule(pool, confFile);
     m_f2 = fopen(m_file_name2, "a+");
     if(m_f == NULL) {
       qerror("could not open file for writing '%s': %s", m_file_name2, strerror(errno));
@@ -1649,7 +1778,7 @@ int main(int argc, const char *const argv[]) {
     nice(10);
     fprintf(stderr, "[%s]: offline mode (writes to %s)\n", cmd, file);
     m_date_str[0] = '\0';
-    readStdinOffline(config);
+    readStdinOffline(pool, config);
     if(!m_verbose) {
       fprintf(stdout, "\n");
     }
@@ -1659,7 +1788,7 @@ int main(int argc, const char *const argv[]) {
      * to write the data every minute. 
      */
     pthread_create(&tid, tha, loggerThread, NULL);
-    readStdin(config);
+    readStdin(pool, config);
   }
   fclose(m_f);
   return 0;
