@@ -7,7 +7,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2012 Pascal Buchbinder
+ * Copyright (C) 2007-2013 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,7 +26,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsrotate.c,v 1.13 2012/09/20 15:19:21 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsrotate.c,v 1.19 2013/04/01 08:31:21 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -65,6 +65,10 @@ static long m_messages = 0;
 static char *m_cmd = NULL;
 static int m_compress = 0;
 static int m_stdout = 0;
+static long m_counter = 0;
+static long m_limit = 2147483648 - (128 * 1024);
+static int m_offset = 0;
+static int m_offset_enabled = 0;
 
 static void usage(char *cmd, int man) {
   if(man) {
@@ -76,12 +80,12 @@ static void usage(char *cmd, int man) {
   if(man) {
     printf(".SH NAME\n");
   }
-  qs_man_print(man, "Log rotation tool (similar to Apache's rotatelogs).\n");
+  qs_man_print(man, "%s - a log rotation tool (similar to Apache's rotatelogs).\n", cmd);
   printf("\n");
   if(man) {
     printf(".SH SYNOPSIS\n");
   }
-  qs_man_print(man, "%s%s -o <file> [-s <sec>] [-f] [-z] [-g <num>] [-u <name>] [-p]\n", man ? "" : "Usage: ", cmd);
+  qs_man_print(man, "%s%s -o <file> [-s <sec> [-t <hours>]] [-f] [-z] [-g <num>] [-u <name>] [-p]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
   if(man) {
     printf(".SH DESCRIPTION\n");
@@ -99,11 +103,18 @@ static void usage(char *cmd, int man) {
   if(man) printf(".TP\n");
   qs_man_print(man, "  -o <file>\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Output log file to write the data to.\n");
+  qs_man_print(man, "     Output log file to write the data to (use absolute path).\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -s <sec>\n");
   if(man) printf("\n");
   qs_man_print(man, "     Rotation interval in seconds, default are 86400 seconds.\n");
+  qs_man_print(man, "  -t <hours>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Offset to UTC (enables also DST support), default is 0.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -b <bytes>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     File size limitation (default are %ld bytes).\n", m_limit);
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -f\n");
   if(man) printf("\n");
@@ -158,6 +169,18 @@ static void usage(char *cmd, int man) {
   }
 }
 
+static time_t get_now() {
+  time_t now = time(NULL);
+  if(m_offset_enabled) {
+    struct tm lcl = *localtime(&now);
+    if(lcl.tm_isdst) {
+      now += 3600;
+    }
+    now += m_offset;
+  }
+  return now;
+}
+
 static int openFile(const char *cmd, const char *file_name) {
   int m_nLogFD = open(file_name, O_WRONLY | O_CREAT | O_APPEND, 0660);
   /* error while opening log file */
@@ -214,19 +237,23 @@ void sigchild(int signo) {
  * Rotates a file
  *
  * @param cmd Command name to be used in log messages
+ * @param now
  * @param file_name Name of the file to rotate (rename)
  * @param messages Error message if rotation was not successful
  */
-static void rotate(const char *cmd, const char *file_name, long *messages) {
+static void rotate(const char *cmd, time_t now,
+		   const char *file_name, long *messages) {
   int rc;
   char arch[HUGE_STR+20];
   char tmb[20];
-  time_t now = time(NULL);
   struct tm *ptr = localtime(&now);
   strftime(tmb, sizeof(tmb), "%Y%m%d%H%M%S", ptr);
   snprintf(arch, sizeof(arch), "%s.%s", file_name, tmb);
+
   /* set next rotation time */
   m_tLogEnd = ((now / m_tRotation) * m_tRotation) + m_tRotation;
+  // reset byte counter
+  m_counter = 0;
   
   /* rename current file */
   if(m_nLogFD >= 0) {
@@ -276,12 +303,12 @@ static void *forcedRotationThread(void *argv) {
   time_t n;
   while(1) {
     qs_csLock();
-    now = time(NULL);
+    now = get_now();
     if(now > m_tLogEnd) {
-      rotate(m_cmd, m_file_name, &m_messages);
+      rotate(m_cmd, now, m_file_name, &m_messages);
     }
     qs_csUnLock();
-    now = time(NULL);
+    now = get_now();
     n = 1 + m_tLogEnd - now;
     sleep(n);
   }
@@ -319,10 +346,20 @@ int main(int argc, char **argv) {
       if (--argc >= 1) {
 	m_tRotation = atoi(*(++argv));
       } 
+    } else if(strcmp(*argv,"-t") == 0) {
+      if (--argc >= 1) {
+	m_offset = atoi(*(++argv));
+	m_offset = m_offset * 3600;
+	m_offset_enabled = 1;
+      } 
     } else if(strcmp(*argv,"-g") == 0) {
       if (--argc >= 1) {
 	m_generations = atoi(*(++argv));
       } 
+    } else if(strcmp(*argv,"-b") == 0) {
+      if (--argc >= 1) {
+	m_limit = atoi(*(++argv));
+      } 
     } else if(strcmp(*argv,"-z") == 0) {
       m_compress = 1;
     } else if(strcmp(*argv,"-p") == 0) {
@@ -344,6 +381,7 @@ int main(int argc, char **argv) {
   }
 
   if(m_file_name == NULL) usage(m_cmd, 0);
+  if(m_limit < (1024 * 1024)) usage(m_cmd, 0);
 
   if(username && getuid() == 0) {
     struct passwd *pwd = getpwnam(username);
@@ -367,7 +405,7 @@ int main(int argc, char **argv) {
   }
   
   /* set next rotation time */
-  now = time(NULL);
+  now = get_now();
   m_tLogEnd = ((now / m_tRotation) * m_tRotation) + m_tRotation;
   /* open file */
   m_nLogFD = openFile(m_cmd, m_file_name);
@@ -388,7 +426,8 @@ int main(int argc, char **argv) {
     if(m_force_rotation) {
       qs_csLock();
     }
-    now = time(NULL);
+    m_counter += nRead;
+    now = get_now();
     /* write data if we have a file handle (else continue but drop log data,
        re-try to open the file at next rotation time) */
     if(m_nLogFD >= 0) {
@@ -411,9 +450,9 @@ int main(int argc, char **argv) {
     } else {
       m_messages++;
     }
-    if(now > m_tLogEnd) {
+    if((now > m_tLogEnd) || (m_counter > m_limit)) {
       /* rotate! */
-      rotate(m_cmd, m_file_name, &m_messages);
+      rotate(m_cmd, now, m_file_name, &m_messages);
     }
     if(m_force_rotation) {
       qs_csUnLock();
@@ -421,4 +460,3 @@ int main(int argc, char **argv) {
   }
   return 0;
 }
-
