@@ -28,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qssign.c,v 1.25 2012/09/20 15:19:21 pbuchbinder Exp $";
+static const char revision[] = "$Id: qssign.c,v 1.28 2013/06/30 18:55:53 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <unistd.h>
@@ -369,7 +369,7 @@ static long qs_verify(const char *sec) {
   int line_size = sizeof(line);
   int line_len;
   m_nr = -1; // sequence number
-  while(fgets(line, sizeof(line), stdin) != NULL) {
+  while(fgets(line, line_size, stdin) != NULL) {
     int valid = 0;
     long ns = 0;
     HMAC_CTX ctx;
@@ -499,6 +499,9 @@ static char *qs_readpwd(apr_pool_t *pool, const char *prg) {
   }
   args[i] = NULL;
 
+  if(cmd == NULL) {
+    qs_failedexec("can't read password, invalid executable", prg, APR_EGENERAL);
+  }
   if((status = apr_procattr_create(&attr, pool)) != APR_SUCCESS) {
     qs_failedexec("while reading password from executable", prg, status);
   }
@@ -545,7 +548,7 @@ static void usage(char *cmd, int man) {
   if(man) {
     printf(".SH NAME\n");
   }
-  qs_man_print(man, "Utility to sign and verify the integrity of log data.\n");
+  qs_man_print(man, "%s - an utility to sign and verify the integrity of log data.\n", cmd);
   printf("\n");
   if(man) {
     printf(".SH SYNOPSIS\n");
@@ -589,7 +592,7 @@ static void usage(char *cmd, int man) {
   } else {
     printf("Example (sign):\n");
   }
-  qs_man_println(man, " TransferLog \"|./bin/%s -s password -e |./bin/qsrotate -o /var/log/apache/access_log\"\n", cmd);
+  qs_man_println(man, " TransferLog \"|/bin/%s -s password -e |/bin/qsrotate -o /var/log/apache/access_log\"\n", cmd);
   printf("\n");
   if(man) {
     printf("\n");
